using System;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Starksoft.Cryptography.OpenPGP;
using System.IO;

namespace OpenPGP_Sample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // load the registered OpenPGP keys in your local key ring to the GridView on the Windows Form
            LoadKeys();

            // set some default data to encrypt
            tbUnencrypted.Text = @"The Starksoft OpenPGP library provides classes that interface with the GPG.EXE command line tool.  The Starksoft OpenPGP libraries allows any .NET application to use GPG.EXE to encrypt or decypt data using .NET IO Streams.  No temporary files are required and everything is handled through streams.  Any .NET Stream object can be used as long as the source stream can be read and the destination stream can be written to.  But, in order for the Starksoft OpenPGP library to work you must first install the lastest version of GnuPG which includes GPG.EXE.  You can obtain the latest version at http://www.gnupg.org/.  See the GPG.EXE tool documentation for information on how to add keys to the GPG key ring and creating your public and private keys.";
        }

        // this method will load the GnuPG OpenPGP keys that are in your local key ring
        // into a GridView control on the Windows Form.
        private void LoadKeys()
        {
            try
            {
                GnuPG gpg = new GnuPG();

                // optionally you can specify the path to the binary gpg.exe program.
                // if the path is not specified the Starksoft OpenPGP library will locate the
                // gpg.exe program using the Windows registry.
                //gpg.BinaryPath = @"C:\Program Files\gnu\GnuPG\gpg.exe";

                // retrieve a collection of keys from the gpg.exe application and convert
                // that collection to a DataTable so we can easily bind it to a GridView control
                gvKeys.DataSource = gpg.GetKeys().ToDataTable();
            }
            catch
            {
                MessageBox.Show("An error occurred while trying access the OpenPGP security keys.\r\n\r\nPlease make sure you have installed Gnu's OpenPGP application\r\nand that the OpenPGP settings are correctly entered.\r\nDownload GnuPG from http:\\www.gnupg.org\r\n\r\nMake sure that you have created a private key using the GPG.EXE application.", "Starksoft OpenPGP Library Sample");
            }
        }

        // retrieve the recipient key that is selected by the user in the GridView control named gvKeys
        private string GetRecipientKey()
        {
            if (gvKeys.SelectedRows.Count == 0)
            {
                MessageBox.Show("Please select a recipient from your key ring.", "Encrypt");
                return "";
            }

            // retrieve the recipient UserId text data that is currently selected in the gvKeys control
            return gvKeys.SelectedRows[0].Cells["UserId"].Value.ToString();
        }


        // this method will encrypt any OpenPGP data found in the TextBox named tbUnencrypted to the TextBox control named tbEncrypted
        private void btnEncrypt_Click(object sender, EventArgs e)
        {
            string recipient = GetRecipientKey();
            // if no recipient is specified then don't do the encryption
            if (recipient.Length == 0)
                return;

            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            MemoryStream unencrypted = new MemoryStream(Encoding.ASCII.GetBytes(tbUnencrypted.Text));
            MemoryStream encrypted = new MemoryStream();
            
            // create a new GnuPG object
            GnuPG gpg = new GnuPG();

            // specify a recipient that is in your key ring
            gpg.Recipient = recipient;

            try
            {
                // encrypt the data using IO Streams - any type of input and output IO Stream can be used
                // as long as the source (input) stream can be read and the destination (output) stream 
                // can be written to
                gpg.Encrypt(unencrypted, encrypted);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Encryption Error");
                return;
            }
            
            // create a stream reader so we can read all the data from the memory stream
            StreamReader reader = new StreamReader(encrypted);
            
            // set the encrypted data stream to the starting position so we can read the entire contents
            encrypted.Position = 0;

            // read the text data from the memory stream.
            tbEncrypted.Text = reader.ReadToEnd();
        }
                
        // this method will decrypt any OpenPGP data found in the TextBox named tbEncrypted to the TextBox control named tbUnencrypted
        private void btnDecrypt_Click(object sender, EventArgs e)
        {
            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            MemoryStream unencrypted = new MemoryStream();
            MemoryStream encrypted = new MemoryStream(Encoding.ASCII.GetBytes(tbEncrypted.Text));

            // create a new GnuPG object
            GnuPG gpg = new GnuPG();
                      
            // specify our secret passphrase (if you have one)
            gpg.Passphrase = tbPassphrase.Text;


            try
            {
                // decrypt the data using IO Streams - any type of input and output IO Stream can be used
                // as long as the source (input) stream can be read and the destination (output) stream 
                // can be written to
                gpg.Decrypt(encrypted, unencrypted);
            }
            catch (GnuPGBadPassphraseException)
            {
                MessageBox.Show("Please enter a valid passphrase.", "Passphrase Needed");
                tbPassphrase.Focus();
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Decryption Error");
                return;
            }

            // create a stream reader so we can read all the data from the memory stream
            StreamReader reader = new StreamReader(unencrypted);

            // set the encrypted data stream to the starting position so we can read the entire contents
            unencrypted.Position = 0;

            // read the text data from the memory stream.
            tbUnencrypted.Text = reader.ReadToEnd();
        }

        // clear any data in the TextBox control named tbUnencrypted
        private void lnkClear_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            tbUnencrypted.Text = "";
        }

        // clear any data in the TextBox control named tbEncrypted
        private void lnkClearEncrypted_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            tbEncrypted.Text = "";
        }

        private void btnSign_Click(object sender, EventArgs e)
        {
            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            MemoryStream unsigned = new MemoryStream(Encoding.ASCII.GetBytes(tbUnencrypted.Text));
            MemoryStream signed = new MemoryStream();

            // create a new GnuPG object
            GnuPG gpg = new GnuPG();

            // specify our secret passphrase (if you have one)
            gpg.Passphrase = tbPassphrase.Text;


            try
            {
                // sign the data using IO Streams - any type of input and output IO Stream can be used
                // as long as the source (input) stream can be read and the destination (output) stream 
                // can be written to
                gpg.Sign(unsigned, signed);
            }
            catch (GnuPGBadPassphraseException)
            {
                MessageBox.Show("Please enter a valid passphrase.", "Passphrase Needed");
                tbPassphrase.Focus();
                return;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Signing Error");
                return;
            }

            // create a stream reader so we can read all the data from the memory stream
            BinaryReader reader = new BinaryReader(signed);

            // set the encrypted data stream to the starting position so we can read the entire contents
            signed.Position = 0;
            tbEncrypted.Text = Convert.ToBase64String(reader.ReadBytes((int)signed.Length));
        }

        private void btnVerify_Click(object sender, EventArgs e)
        {
            // create two memory stream - one to hold the unencrypted data and the other 
            // stream holds the encrypted data.  We can use any System.IO stream such as 
            // FileStream but for this demo we will use a memory stream
            MemoryStream signed = new MemoryStream(Convert.FromBase64String(tbEncrypted.Text));

            // create a new GnuPG object
            GnuPG gpg = new GnuPG();

            try
            {
                // sign the data using IO Streams - any type of input and output IO Stream can be used
                // as long as the source (input) stream can be read and the destination (output) stream 
                // can be written to
                gpg.Verify(signed);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Signing Error");
                return;
            }

            MessageBox.Show("verified");
        }


    }
}